// CustomStudyReqDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Common.h"
#include "Helper.h"
#include "CustomStudy.h"
#include "CustomStudyReqDlg.h"

namespace
{
   VARIANT_BOOL boolToVarBool(BOOL value)
   {
      return value ? VARIANT_TRUE : VARIANT_FALSE;
   }
}

CCustomStudyReqDlg::CCustomStudyReqDlg(
                              const ATL::CComPtr<ICQGCustomStudyRequest>& spRequest,
                              const DATE lineTime,
                              CWnd* pParent /*=NULL*/)
                              : CDialog(CCustomStudyReqDlg::IDD, pParent),
                                m_spRequest(spRequest),
                                m_LineTime(lineTime),
                                m_BaseExpression(_T("")),
                                m_RangeStartDate(COleDateTime::GetCurrentTime()),
                                m_RangeEndDate(COleDateTime::GetCurrentTime()),
                                m_RequestType(rtIntInt),
                                m_EqualizeCloses(FALSE),
                                m_UpdatesEnabled(FALSE),
                                m_DailyFromIntraday(FALSE),
                                m_CustomSession(FALSE)
{
}

void CCustomStudyReqDlg::DoDataExchange(
                              CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);

   DDX_Text(pDX, IDC_ED_BASE_EXPRESSION, m_BaseExpression);

   DDX_Control(pDX, IDC_LSV_PARAMETERS, m_lvParameters);
   DDX_Control(pDX, IDC_ED_PARAMETER_VALUE, m_edParameterValue);
   DDX_Control(pDX, IDC_BTN_CHANGE_VALUE, m_btnChangeValue);

   DDX_Control(pDX, IDC_DTP_RANGE_START, m_dtpRangeStart);
   DDX_DateTimeCtrl(pDX, IDC_DTP_RANGE_START, m_RangeStartDate);
   DDX_Control(pDX, IDC_ED_RANGE_START, m_edRangeStart);
   DDX_Control(pDX, IDC_SPN_RANGE_START, m_spnRangeStart);
   
   DDX_Control(pDX, IDC_DTP_RANGE_END, m_dtpRangeEnd);
   DDX_DateTimeCtrl(pDX, IDC_DTP_RANGE_END, m_RangeEndDate);
   DDX_Control(pDX, IDC_ED_RANGE_END, m_edRangeEnd);
   DDX_Control(pDX, IDC_SPN_RANGE_END, m_spnRangeEnd);

   DDX_Radio(pDX, IDC_RD_INT_INT, m_RequestType);

   DDX_Control(pDX, IDC_CHK_INCLUDE_END, m_chkIncludeEnd);
   DDX_Check(pDX, IDC_CHK_INCLUDE_END, m_IncludeEnd);

   DDX_Control(pDX, IDC_CMB_HISTORICAL, m_cmbHistorical);

   DDX_Control(pDX, IDC_ED_INTRADAY, m_edIntraday);
   DDX_Control(pDX, IDC_SPN_INTRADAY, m_spnIntraday);

   DDX_Control(pDX, IDC_CMB_CONTINUATION, m_cmbContinuation);

   DDX_Control(pDX, IDC_CHK_EQUALIZE_CLOSES, m_chkEqualizeCloses);
   DDX_Check(pDX, IDC_CHK_EQUALIZE_CLOSES, m_EqualizeCloses);

   DDX_Control(pDX, IDC_ED_DAYS_BEFORE_EXP, m_edDaysBeforeExp);
   DDX_Control(pDX, IDC_SPN_DAYS_BEFORE_EXP, m_spnDaysBeforeExp);

   DDX_Check(pDX, IDC_CHK_UPDATES_ENABLED, m_UpdatesEnabled);

   DDX_Check(pDX, IDC_CHK_DAILY_FROM_INTR, m_DailyFromIntraday);

   DDX_Control(pDX, IDC_ED_SESSIONS_FILTER, m_edSessionsFilter);
   DDX_Control(pDX, IDC_SPN_SESSIONS_FILTER, m_spnSessionsFilter);
   
   DDX_Control(pDX, IDC_ED_CUSTOM_SESSIONS_FILTER, m_edCustomSessionsFilter);
   DDX_Text(pDX, IDC_ED_CUSTOM_SESSIONS_FILTER, m_CustomSessionsFilter);

   DDX_Check(pDX, IDC_CHK_CUSTOM_SESSION, m_CustomSession);

   DDX_Control(pDX, IDC_LSV_CURVES, m_lvCurves);
   
   DDX_Control(pDX, IDC_CMB_SUBSCRIPTION, m_cmbSubscription);
   
   DDX_Control(pDX, IDC_ST_RECALC_PERIOD, m_lblRecalcPeriod);
   DDX_Control(pDX, IDC_ED_RECALC_PERIOD, m_edRecalcPeriod);
   DDX_Control(pDX, IDC_SPN_RECALC_PERIOD, m_spnRecalcPeriod);
}

BOOL CCustomStudyReqDlg::OnInitDialog()
{
   try
   {
      CDialog::OnInitDialog();

      InitializeControls();
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return TRUE;
}

BEGIN_MESSAGE_MAP(CCustomStudyReqDlg, CDialog)
   ON_BN_CLICKED(IDOK, OnBnClickedOk)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_SESSIONS_FILTER, OnDeltaposSpnSessionsFilter)
   ON_BN_CLICKED(IDC_CHK_CUSTOM_SESSION, OnBnClickedChkCustomSession)
   ON_CBN_SELCHANGE(IDC_CMB_CONTINUATION, OnCbnSelChangeCmbContinuation)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_INTRADAY, OnDeltaposSpnIntraday)
   ON_CBN_SELCHANGE(IDC_CMB_HISTORICAL, OnCbnSelChangeCmbHistorical)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_RANGE_END, OnDeltaposSpnRangeEnd)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_RANGE_START, OnDeltaposSpnRangeStart)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_DAYS_BEFORE_EXP, OnDeltaposSpnDaysBeforeExp)
   ON_BN_CLICKED(IDC_RD_INT_INT, OnBnClickedRdRequestType)
   ON_BN_CLICKED(IDC_RD_DATE_INT, OnBnClickedRdRequestType)
   ON_BN_CLICKED(IDC_RD_DATE_DATE, OnBnClickedRdRequestType)
   ON_BN_CLICKED(IDC_BTN_CHANGE_VALUE, OnBnClickedBtnChangeValue)
   ON_NOTIFY(LVN_ITEMCHANGED, IDC_LSV_PARAMETERS, OnLvnItemchangedLsvParameters)
   ON_BN_CLICKED(IDC_BTN_SELECT_ALL_CURVES, OnBnClickedBtnSelectAllCurves)
   ON_BN_CLICKED(IDC_BTN_UNSELECT_ALL_CURVES, OnBnClickedBtnUnselectAllCurves)
   ON_CBN_SELCHANGE(IDC_CMB_SUBSCRIPTION, OnCbnSelChangeCmbSubscription)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_RECALC_PERIOD, OnDeltaposSpnRecalcPeriod)
END_MESSAGE_MAP()

void CCustomStudyReqDlg::InitializeControls()
{
   m_edRangeStart.SetNegativeAllowed(true);
   m_edRangeEnd.SetNegativeAllowed(true);
   m_edParameterValue.SetNegativeAllowed(true);
   m_edParameterValue.SetValue(0);
   m_edParameterValue.SetMaxFractionalPartLength(CHelper::PARAM_FRAC_PART_LENGTH);

   RequestTypeChanged();

   InitializeParametersList();
   InitializeCurvesList();

   InitializeHistoricalCombo();
   InitializeContinuationCombo();
   InitializeSubscriptionCombo();

   m_dtpRangeStart.SetFormat(CHelper::DATE_FORMAT);
   m_dtpRangeEnd.SetFormat(CHelper::DATE_FORMAT);

   // Set default parameters
   SetDefaultParameters();

   AddParameters();
   AddCurves();
}

void CCustomStudyReqDlg::InitializeParametersList()
{
   m_lvParameters.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);
   m_lvParameters.ModifyStyle(0, LVS_SHOWSELALWAYS | LVS_SINGLESEL);

   m_lvParameters.InsertColumn(pcName, _T("Name"), LVCFMT_LEFT, 80);
   m_lvParameters.InsertColumn(pcComment, _T("Comment"), LVCFMT_LEFT, 160);
   m_lvParameters.InsertColumn(pcType, _T("Type"), LVCFMT_LEFT, 50);
   m_lvParameters.InsertColumn(pcDefaultValue, _T("Def. Value"), LVCFMT_LEFT, 70);
   m_lvParameters.InsertColumn(pcValue, _T("Curr. Value"), LVCFMT_LEFT, 70);
}

void CCustomStudyReqDlg::InitializeCurvesList()
{
   m_lvCurves.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_CHECKBOXES);
   m_lvCurves.ModifyStyle(0, LVS_SHOWSELALWAYS | LVS_SINGLESEL);

   m_lvCurves.InsertColumn(ccCheck, _T("#"), LVCFMT_LEFT, 20);
   m_lvCurves.InsertColumn(ccName, _T("Name"), LVCFMT_LEFT, 110);
}

void CCustomStudyReqDlg::SetDefaultParameters()
{
   UpdateData();

   // Get default values form request
   long defIntradayPeriod;
   HRESULT hr = m_spRequest->get_IntradayPeriod(&defIntradayPeriod);
   Common::ThrowOnCOMError(m_spRequest, hr);

   eHistoricalPeriod defHistoricalPeriod;
   hr = m_spRequest->get_HistoricalPeriod(&defHistoricalPeriod);
   Common::ThrowOnCOMError(m_spRequest, hr);

   VARIANT_BOOL defUpdatesEnabled;
   hr = m_spRequest->get_UpdatesEnabled(&defUpdatesEnabled);
   Common::ThrowOnCOMError(m_spRequest, hr);

   VARIANT_BOOL defIncludeEnd;
   hr = m_spRequest->get_IncludeEnd(&defIncludeEnd);
   Common::ThrowOnCOMError(m_spRequest, hr);

   ATL::CComVariant defSessionsFilter;
   hr = m_spRequest->get_SessionsFilter(&defSessionsFilter);
   Common::ThrowOnCOMError(m_spRequest, hr);

   eTimeSeriesContinuationType defContinuation;
   hr = m_spRequest->get_Continuation(&defContinuation);
   Common::ThrowOnCOMError(m_spRequest, hr);

   VARIANT_BOOL defEqualizeCloses;
   hr = m_spRequest->get_EqualizeCloses(&defEqualizeCloses);
   Common::ThrowOnCOMError(m_spRequest, hr);

   long defDaysBeforeExp;
   hr = m_spRequest->get_DaysBeforeExpiration(&defDaysBeforeExp);
   Common::ThrowOnCOMError(m_spRequest, hr);

   eSessionFlag defSessionFlags;
   hr = m_spRequest->get_SessionFlags(&defSessionFlags);
   Common::ThrowOnCOMError(m_spRequest, hr);
   
   eTimeSeriesSubscriptionLevel defSubscription;
   hr = m_spRequest->get_SubscriptionLevel(&defSubscription);
   Common::ThrowOnCOMError(m_spRequest, hr);
   
   long defRecalcPeriod;
   hr = m_spRequest->get_RecalcPeriod(&defRecalcPeriod);
   Common::ThrowOnCOMError(m_spRequest, hr);

   // Set controls default values
   m_edIntraday.SetValue(defIntradayPeriod);
   CHelper::SetComboBoxItem(m_cmbHistorical, (DWORD_PTR)defHistoricalPeriod);
   m_UpdatesEnabled = defUpdatesEnabled;
   m_IncludeEnd = defIncludeEnd;
   CHelper::SetComboBoxItem(m_cmbContinuation, (DWORD_PTR)defContinuation);
   m_EqualizeCloses = defEqualizeCloses;
   m_edDaysBeforeExp.SetValue(defDaysBeforeExp);
   CHelper::SetComboBoxItem(m_cmbSubscription, (DWORD_PTR)defSubscription);
   m_edRecalcPeriod.SetValue(defRecalcPeriod);
   SetSessionFlags(defSessionFlags);
   if (defSessionsFilter.vt == VT_UI4)
   {
      m_CustomSession = FALSE;
      m_edSessionsFilter.SetValue(defSessionsFilter.intVal);
      m_CustomSessionsFilter = _T("");
   }
   else if (defSessionsFilter.vt == VT_BSTR)
   {
      m_CustomSession = TRUE;
      m_edSessionsFilter.SetValue(0);
      m_CustomSessionsFilter = defSessionsFilter.bstrVal;
   }
   OnCbnSelChangeCmbHistorical();
   OnCbnSelChangeCmbContinuation();
   OnCbnSelChangeCmbSubscription();
   OnBnClickedChkCustomSession();

   m_edRangeStart.SetValue(0);
   m_RangeStartDate = m_LineTime - 1;
   m_edRangeEnd.SetValue(-10);
   m_RangeEndDate = m_LineTime;

   UpdateData(FALSE);
}

void CCustomStudyReqDlg::AddParameters()
{
   // Get definition from request
   ATL::CComPtr<ICQGCustomStudyDefinition> spCustomStudyDefinition;
   HRESULT hr = m_spRequest->get_Definition(&spCustomStudyDefinition);
   Common::ThrowOnCOMError(m_spRequest, hr);

   // Add all parameters into the list
   m_lvParameters.DeleteAllItems();

   ATL::CComPtr<ICQGParameterDefinitions> spParameterDefinitions;
   hr = spCustomStudyDefinition->get_ParameterDefinitions(&spParameterDefinitions);
   Common::ThrowOnCOMError(spCustomStudyDefinition, hr);

   long count;
   hr = spParameterDefinitions->get_Count(&count);
   Common::ThrowOnCOMError(spParameterDefinitions, hr);

   for (long i = 0; i < count; ++i)
   {
      ATL::CComPtr<ICQGParameterDefinition> spParameterDefinition;
      hr = spParameterDefinitions->get_Item(i, &spParameterDefinition);
      Common::ThrowOnCOMError(spParameterDefinitions, hr);

      ATL::CComBSTR name;
      hr = spParameterDefinition->get_Name(&name);
      Common::ThrowOnCOMError(spParameterDefinition, hr);

      ATL::CComBSTR comment;
      hr = spParameterDefinition->get_Comment(&comment);
      Common::ThrowOnCOMError(spParameterDefinition, hr);

      eUserFormulaParameterType parameterType;
      hr = spParameterDefinition->get_Type(&parameterType);
      Common::ThrowOnCOMError(spParameterDefinition, hr);

      ATL::CComVariant defaultValue;
      hr = spParameterDefinition->get_DefaultValue(&defaultValue);
      Common::ThrowOnCOMError(spParameterDefinition, hr);

      const int row = m_lvParameters.InsertItem(m_lvParameters.GetItemCount(), _T(""));
      m_lvParameters.SetItemText(row, pcName, ATL::CW2T(name));
      m_lvParameters.SetItemText(row, pcComment, ATL::CW2T(comment));
      m_lvParameters.SetItemText(row, pcType, CHelper::ParameterTypeToString(parameterType));
      m_lvParameters.SetItemText(row, pcDefaultValue,
                        Common::VariantToString(defaultValue, CHelper::PARAM_FRAC_PART_LENGTH));
      m_lvParameters.SetItemText(row, pcValue,
                        Common::VariantToString(defaultValue, CHelper::PARAM_FRAC_PART_LENGTH));
   }
}

void CCustomStudyReqDlg::AddCurves()
{
   // Get definition from request
   ATL::CComPtr<ICQGCustomStudyDefinition> spCustomStudyDefinition;
   HRESULT hr = m_spRequest->get_Definition(&spCustomStudyDefinition);
   Common::ThrowOnCOMError(m_spRequest, hr);

   // Add all curves into the list
   m_lvCurves.DeleteAllItems();

   ATL::CComPtr<ICQGCurveDefinitions> spCurveDefinitions;
   hr = spCustomStudyDefinition->get_CurveDefinitions(&spCurveDefinitions);
   Common::ThrowOnCOMError(spCustomStudyDefinition, hr);

   long count;
   hr = spCurveDefinitions->get_Count(&count);
   Common::ThrowOnCOMError(spCurveDefinitions, hr);

   for (long i = 0; i < count; ++i)
   {
      ATL::CComPtr<ICQGCurveDefinition> spCurveDefinition;
      hr = spCurveDefinitions->get_Item(i, &spCurveDefinition);
      Common::ThrowOnCOMError(spCurveDefinitions, hr);

      ATL::CComBSTR name;
      hr = spCurveDefinition->get_Name(&name);
      Common::ThrowOnCOMError(spCustomStudyDefinition, hr);

      const int row = m_lvCurves.InsertItem(m_lvCurves.GetItemCount(), _T(""));
      m_lvCurves.SetItemText(row, ccName, ATL::CW2T(name));
      m_lvCurves.SetCheck(row, TRUE);
   }
}

void CCustomStudyReqDlg::OnBnClickedOk()
{
   try
   {
      UpdateData();

      // Set request parameters
      ATL::CComBSTR baseExpression(m_BaseExpression);
      HRESULT hr = m_spRequest->put_BaseExpression(baseExpression);
      Common::ThrowOnCOMError(m_spRequest, hr);

      if (m_RequestType == rtIntInt)
      {
         hr = m_spRequest->put_RangeStart(ATL::CComVariant(m_edRangeStart.GetValue()));
      }
      else
      {
         hr = m_spRequest->put_RangeStart(ATL::CComVariant(m_RangeStartDate.m_dt, VT_DATE));
      }
      Common::ThrowOnCOMError(m_spRequest, hr);

      if (m_RequestType == rtDateDate)
      {
         hr = m_spRequest->put_RangeEnd(ATL::CComVariant(m_RangeEndDate.m_dt, VT_DATE));
      }
      else
      {
         hr = m_spRequest->put_RangeEnd(ATL::CComVariant(m_edRangeEnd.GetValue()));
      }
      Common::ThrowOnCOMError(m_spRequest, hr);

      hr = m_spRequest->put_IncludeEnd(boolToVarBool(m_IncludeEnd));
      Common::ThrowOnCOMError(m_spRequest, hr);

      // Get historical period
      eHistoricalPeriod historicalPeriod = 
            static_cast<eHistoricalPeriod>(m_cmbHistorical.GetItemData(m_cmbHistorical.GetCurSel()));

      if (historicalPeriod == hpUndefined)
      {
         hr = m_spRequest->put_IntradayPeriod(m_edIntraday.GetValue());
      }
      else
      {
         hr = m_spRequest->put_HistoricalPeriod(historicalPeriod);
      }
      Common::ThrowOnCOMError(m_spRequest, hr);

      // Get continuation type
      eTimeSeriesContinuationType continuationType = 
         static_cast<eTimeSeriesContinuationType>(m_cmbContinuation.GetItemData(m_cmbContinuation.GetCurSel()));

      hr = m_spRequest->put_Continuation(continuationType);
      Common::ThrowOnCOMError(m_spRequest, hr);

      hr = m_spRequest->put_EqualizeCloses(boolToVarBool(m_EqualizeCloses));
      Common::ThrowOnCOMError(m_spRequest, hr);

      hr = m_spRequest->put_DaysBeforeExpiration(m_edDaysBeforeExp.GetValue());
      Common::ThrowOnCOMError(m_spRequest, hr);

      hr = m_spRequest->put_UpdatesEnabled(boolToVarBool(m_UpdatesEnabled));
      Common::ThrowOnCOMError(m_spRequest, hr);

      if (m_CustomSession)
      {
         hr = m_spRequest->put_SessionsFilter(ATL::CComVariant(m_CustomSessionsFilter));
      }
      else
      {
         hr = m_spRequest->put_SessionsFilter(ATL::CComVariant(m_edSessionsFilter.GetValue()));
      }
      Common::ThrowOnCOMError(m_spRequest, hr);

      hr = m_spRequest->put_SessionFlags(GetSessionFlags());
      Common::ThrowOnCOMError(m_spRequest, hr);

      IncludeCurves();

      // Get subscription type
      eTimeSeriesSubscriptionLevel subscriptionLevel = 
         static_cast<eTimeSeriesSubscriptionLevel>(m_cmbSubscription.GetItemData(m_cmbSubscription.GetCurSel()));

      hr = m_spRequest->put_SubscriptionLevel(subscriptionLevel);
      Common::ThrowOnCOMError(m_spRequest, hr);

      hr = m_spRequest->put_RecalcPeriod(m_edRecalcPeriod.GetValue());
      Common::ThrowOnCOMError(m_spRequest, hr);

      // Close request window
      OnOK();
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CCustomStudyReqDlg::IncludeCurves()
{
   int count = m_lvCurves.GetItemCount();
   bool isCurveSelected = false;

   for (int i = 0; i < count; ++i)
   {
      if (m_lvCurves.GetCheck(i))
      {
         CString curveName = m_lvCurves.GetItemText(i, ccName);
         m_spRequest->IncludeCurve(ATL::CComVariant(curveName));
         isCurveSelected = true;
      }
   }

   if (!isCurveSelected)
   {
      throw std::runtime_error("At least one curve must be selected.");
   }
}

void CCustomStudyReqDlg::CheckAllCurves(
                              BOOL check)
{
   int count = m_lvCurves.GetItemCount();
   for (int i = 0; i < count; ++i)
   {
      m_lvCurves.SetCheck(i, check);
   }
}

void CCustomStudyReqDlg::InitializeHistoricalCombo()
{
   int row = m_cmbHistorical.AddString(_T("Undefined"));
   m_cmbHistorical.SetItemData(row, hpUndefined);

   row = m_cmbHistorical.AddString(_T("Daily"));
   m_cmbHistorical.SetItemData(row, hpDaily);

   row = m_cmbHistorical.AddString(_T("Weekly"));
   m_cmbHistorical.SetItemData(row, hpWeekly);

   row = m_cmbHistorical.AddString(_T("Monthly"));
   m_cmbHistorical.SetItemData(row, hpMonthly);

   row = m_cmbHistorical.AddString(_T("Quarterly"));
   m_cmbHistorical.SetItemData(row, hpQuarterly);

   row = m_cmbHistorical.AddString(_T("Semiannual"));
   m_cmbHistorical.SetItemData(row, hpSemiannual);

   row = m_cmbHistorical.AddString(_T("Yearly"));
   m_cmbHistorical.SetItemData(row, hpYearly);
}

void CCustomStudyReqDlg::InitializeContinuationCombo()
{
   int row = m_cmbContinuation.AddString(_T("NoContinuation"));
   m_cmbContinuation.SetItemData(row, tsctNoContinuation);

   row = m_cmbContinuation.AddString(_T("Standard"));
   m_cmbContinuation.SetItemData(row, tsctStandard);

   row = m_cmbContinuation.AddString(_T("StandardByMonth"));
   m_cmbContinuation.SetItemData(row, tsctStandardByMonth);

   row = m_cmbContinuation.AddString(_T("Active"));
   m_cmbContinuation.SetItemData(row, tsctActive);

   row = m_cmbContinuation.AddString(_T("ActiveByMonth"));
   m_cmbContinuation.SetItemData(row, tsctActiveByMonth);

   row = m_cmbContinuation.AddString(_T("Adjusted"));
   m_cmbContinuation.SetItemData(row, tsctAdjusted);

   row = m_cmbContinuation.AddString(_T("AdjustedByMonth"));
   m_cmbContinuation.SetItemData(row, tsctAdjustedByMonth);
}

void CCustomStudyReqDlg::InitializeSubscriptionCombo()
{
   int row = m_cmbSubscription.AddString(_T("EachBar"));
   m_cmbSubscription.SetItemData(row, tslEachBar);

   row = m_cmbSubscription.AddString(_T("EachTick"));
   m_cmbSubscription.SetItemData(row, tslEachTick);

   row = m_cmbSubscription.AddString(_T("EndOfBarAndPeriod"));
   m_cmbSubscription.SetItemData(row, tslEndOfBarAndPeriod);
}

void CCustomStudyReqDlg::OnBnClickedBtnChangeValue()
{
   try
   {
      // Get parameter name by row
      const int row = m_lvParameters.GetSelectionMark();
      CString name = m_lvParameters.GetItemText(row, pcName);

      // Set parameter into the request
      HRESULT hr = m_spRequest->put_Parameter(ATL::CComVariant(name), m_edParameterValue.GetVariantValue());
      Common::ThrowOnCOMError(m_spRequest, hr);

      // Change parameter value in the list
      m_lvParameters.SetItemText(row, pcValue, 
            Common::VariantToString(m_edParameterValue.GetVariantValue(), CHelper::PARAM_FRAC_PART_LENGTH));
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CCustomStudyReqDlg::OnBnClickedRdRequestType()
{
   try
   {
      RequestTypeChanged();
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CCustomStudyReqDlg::OnCbnSelChangeCmbHistorical()
{
   try
   {
      // Get historical period
      eHistoricalPeriod historicalPeriod = 
         static_cast<eHistoricalPeriod>(m_cmbHistorical.GetItemData(m_cmbHistorical.GetCurSel()));

      // Enable/disable intraday period controls 
      m_edIntraday.EnableWindow(historicalPeriod == hpUndefined);
      m_spnIntraday.EnableWindow(historicalPeriod == hpUndefined);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CCustomStudyReqDlg::OnDeltaposSpnIntraday(
                              NMHDR *pNMHDR,
                              LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edIntraday.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CCustomStudyReqDlg::OnDeltaposSpnRangeStart(
                              NMHDR *pNMHDR,
                              LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edRangeStart.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CCustomStudyReqDlg::OnDeltaposSpnRangeEnd(
                              NMHDR *pNMHDR,
                              LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edRangeEnd.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CCustomStudyReqDlg::OnDeltaposSpnDaysBeforeExp(
                              NMHDR *pNMHDR,
                              LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edDaysBeforeExp.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CCustomStudyReqDlg::OnDeltaposSpnSessionsFilter(
                              NMHDR *pNMHDR,
                              LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edSessionsFilter.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CCustomStudyReqDlg::OnBnClickedChkCustomSession()
{
   try
   {
      UpdateData();

      m_spnSessionsFilter.ShowWindow(m_CustomSession ? SW_HIDE : SW_SHOW);
      m_edSessionsFilter.ShowWindow(m_CustomSession ? SW_HIDE : SW_SHOW);

      m_edCustomSessionsFilter.ShowWindow(m_CustomSession ? SW_SHOW : SW_HIDE);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CCustomStudyReqDlg::OnCbnSelChangeCmbContinuation()
{
   try
   {
      // Get continuation type
      eTimeSeriesContinuationType continuationType = 
         static_cast<eTimeSeriesContinuationType>(m_cmbContinuation.GetItemData(m_cmbContinuation.GetCurSel()));

      switch (continuationType)
      {
      case tsctNoContinuation:
      case tsctStandard:
      case tsctStandardByMonth:
         m_chkEqualizeCloses.EnableWindow(FALSE);
         m_edDaysBeforeExp.EnableWindow(FALSE);
         m_spnDaysBeforeExp.EnableWindow(FALSE);
         break;
      case tsctActive:
      case tsctActiveByMonth:
         m_chkEqualizeCloses.EnableWindow(TRUE);
         m_edDaysBeforeExp.EnableWindow(FALSE);
         m_spnDaysBeforeExp.EnableWindow(FALSE);
         break;
      default:
         m_chkEqualizeCloses.EnableWindow(TRUE);
         m_edDaysBeforeExp.EnableWindow(TRUE);
         m_spnDaysBeforeExp.EnableWindow(TRUE);
      }
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CCustomStudyReqDlg::RequestTypeChanged()
{
   UpdateData();

   m_dtpRangeStart.ShowWindow(m_RequestType == rtIntInt ? SW_HIDE : SW_SHOW);
   m_edRangeStart.ShowWindow(m_RequestType == rtIntInt ? SW_SHOW : SW_HIDE);
   m_spnRangeStart.ShowWindow(m_RequestType == rtIntInt ? SW_SHOW : SW_HIDE);

   m_dtpRangeEnd.ShowWindow(m_RequestType == rtDateDate ? SW_SHOW : SW_HIDE);
   m_edRangeEnd.ShowWindow(m_RequestType == rtDateDate ? SW_HIDE : SW_SHOW);
   m_spnRangeEnd.ShowWindow(m_RequestType == rtDateDate ? SW_HIDE : SW_SHOW);

   m_chkIncludeEnd.EnableWindow(m_RequestType == rtDateDate);
}

void CCustomStudyReqDlg::SetSessionFlags(
                              eSessionFlag sessionFlag)
{
   UpdateData();
   m_DailyFromIntraday = ((sessionFlag & sfDailyFromIntraday) == sfDailyFromIntraday);
   UpdateData(FALSE);
}

eSessionFlag CCustomStudyReqDlg::GetSessionFlags()
{
   UpdateData();

   int sessionFlag = sfUndefined;

   if (m_DailyFromIntraday)
   {
      sessionFlag = (sessionFlag | sfDailyFromIntraday);
   }

   return (eSessionFlag)sessionFlag;
}

void CCustomStudyReqDlg::OnLvnItemchangedLsvParameters(
                              NMHDR *pNMHDR,
                              LRESULT *pResult)
{
   try
   {
      // Disable parameter change controls and set value to 0
      m_btnChangeValue.EnableWindow(false);
      m_edParameterValue.EnableWindow(false);
      m_edParameterValue.SetVariantValue(0);

      LPNMLISTVIEW pNMLV = reinterpret_cast<LPNMLISTVIEW>(pNMHDR);

      if ((pNMLV->uNewState & LVIS_SELECTED) != LVIS_SELECTED)
      {
         // If there is no new item selected then do nothing
         return;
      }

      // Get parameter name
      CString name = m_lvParameters.GetItemText(pNMLV->iItem, pcName);

      // Get parameter current value from request
      ATL::CComVariant value;
      HRESULT hr = m_spRequest->get_Parameter(ATL::CComVariant(name), &value);
      Common::ThrowOnCOMError(m_spRequest, hr);

      // Set parameter current value in the edit control
      m_edParameterValue.SetVariantValue(value);

      // Enable parameter change controls
      m_btnChangeValue.EnableWindow(true);
      m_edParameterValue.EnableWindow(true);

      *pResult = 0;
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CCustomStudyReqDlg::OnBnClickedBtnSelectAllCurves()
{
   try
   {
      CheckAllCurves(TRUE);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CCustomStudyReqDlg::OnBnClickedBtnUnselectAllCurves()
{
   try
   {
      CheckAllCurves(FALSE);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CCustomStudyReqDlg::OnCbnSelChangeCmbSubscription()
{
    try
   {
      // Get subscription level
      eTimeSeriesSubscriptionLevel subscriptionLevel = 
         static_cast<eTimeSeriesSubscriptionLevel>(m_cmbSubscription.GetItemData(m_cmbSubscription.GetCurSel()));

      if (subscriptionLevel == tslEndOfBarAndPeriod)
      {
         m_lblRecalcPeriod.EnableWindow(TRUE);
         m_edRecalcPeriod.EnableWindow(TRUE);
         m_spnRecalcPeriod.EnableWindow(TRUE);
      }
      else
      {
         m_lblRecalcPeriod.EnableWindow(FALSE);
         m_edRecalcPeriod.EnableWindow(FALSE);
         m_spnRecalcPeriod.EnableWindow(FALSE);
      }
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}

void CCustomStudyReqDlg::OnDeltaposSpnRecalcPeriod(
                                NMHDR *pNMHDR,
                                LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edRecalcPeriod.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}