/*
* =====================================================================
*  Filename: CustomStudyReqDlg.h
*    Author: Sevada Gyurjyan
*   Company: CQG, Inc.
* Copyright: (c)2008 by CQG,Inc. All rights reserved. 
* =====================================================================
*/

#pragma once

#include "afxwin.h"
#include "afxdtctl.h"
#include "NumEdit.h"

// CCustomStudyReqDlg dialog
class CCustomStudyReqDlg : public CDialog
{

public:

   /// \brief CCustomStudyReqDlg constructor.
   /// \param spRequest - ICQGCustomStudyRequest object
   /// \param lineTime - line time from CQGCEL
   /// \param pParent - parent window
   CCustomStudyReqDlg(
               const ATL::CComPtr<ICQGCustomStudyRequest>& spRequest,
               const DATE lineTime,
               CWnd* pParent = NULL);

   /// \enum Dialog Data.
   enum { IDD = IDD_CUSTOM_STUDY_REQ_DIALOG };

private:

   /// \brief DDX/DDV support.
   virtual void DoDataExchange(
               CDataExchange* pDX);

   /// \brief This message is sent to the dialog box immediately before the dialog box is displayed.
   /// \return Specifies whether the framework shall set the input 
   /// \return focus to the first control of the dialog box
   virtual BOOL OnInitDialog();

private:

   /// \brief Sets request parameters and closes the form.
   afx_msg void OnBnClickedOk();

   /// \brief Used to increment or decrement integer range start.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnDeltaposSpnRangeStart(
               NMHDR *pNMHDR,
               LRESULT *pResult);

   /// \brief Used to increment or decrement integer range end.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnDeltaposSpnRangeEnd(
               NMHDR *pNMHDR,
               LRESULT *pResult);

   /// \brief Used to increment or decrement intra day period.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnDeltaposSpnIntraday(
               NMHDR *pNMHDR,
               LRESULT *pResult);

   /// \brief Used to increment or decrement days before expiration.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnDeltaposSpnDaysBeforeExp(
               NMHDR *pNMHDR,
               LRESULT *pResult);

   /// \brief Used to increment or decrement sessions filter.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnDeltaposSpnSessionsFilter(
               NMHDR *pNMHDR,
               LRESULT *pResult);

   /// \brief Changes historical type and enables/disables relative controls.
   afx_msg void OnCbnSelChangeCmbHistorical();

   /// \brief Changes request type and unhides/hides relative controls.
   afx_msg void OnBnClickedRdRequestType();

   /// \brief Checks/unchecks custom session and unhides/hides relative controls.
   afx_msg void OnBnClickedChkCustomSession();

   /// \brief Changes continuation type and enables/disables relative controls.
   afx_msg void OnCbnSelChangeCmbContinuation();

   /// \brief Changes parameter's value to the given in the list.
   afx_msg void OnBnClickedBtnChangeValue();
   
   /// \brief Changes selected parameter.
   afx_msg void OnLvnItemchangedLsvParameters(
               NMHDR *pNMHDR, 
               LRESULT *pResult);

   /// \brief Selects all curves.
   afx_msg void OnBnClickedBtnSelectAllCurves();

   /// \brief Unselects all curves.
   afx_msg void OnBnClickedBtnUnselectAllCurves();
   
   /// \brief Changes subscription level and enables/disables relative controls.
   afx_msg void OnCbnSelChangeCmbSubscription();
   
   /// \brief Used to increment or decrement recalc period.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnDeltaposSpnRecalcPeriod(
               NMHDR *pNMHDR,
               LRESULT *pResult);

   /// \brief Declares that the class defines a message map.
   DECLARE_MESSAGE_MAP()

private:

   /// \brief Initializes all controls.
   void InitializeControls();

   /// \brief Initializes parameters list.
   void InitializeParametersList();

   /// \brief Initializes curves list.
   void InitializeCurvesList();

   /// \brief Sets all parameters default values taken from CQGCustomStudyRequest.
   void SetDefaultParameters();

   /// \brief Adds parameters into the list.
   void AddParameters();

   /// \brief Adds curves into the list.
   void AddCurves();

   /// \brief Initializes historical periods combo box.
   void InitializeHistoricalCombo();

   /// \brief Initializes continuation types combo box.
   void InitializeContinuationCombo();
   
   /// \brief Initializes subscription combo box.
   void InitializeSubscriptionCombo();
   
   /// \brief Changes the request type.
   void RequestTypeChanged();

   /// \brief Checks fields in the form according to the specified session flags bit mask.
   /// \param sessionFlag - sessions flag bit mask
   void SetSessionFlags(
               eSessionFlag sessionFlag);

   /// \brief Returns session flag bit mask according to the checked fields.
   /// \return Session flag as eSessionFlag object.
   eSessionFlag GetSessionFlags();

   /// \brief Adds all selected curves to the request.
   void IncludeCurves();

   /// \brief Checks/Unchecks all curves.
   /// \param check - Check status
   void CheckAllCurves(
               BOOL check);

private:

   /// \enum eRequestType. Represents request type.
   enum eRequestType;

   /// \enum eParametersColumns. Represents parameters columns in the list.
   enum eParametersColumns;

   /// \enum eCurvesColumns. Represents curves columns in the list.
   enum eCurvesColumns;

private:

   /// \brief CQGCustomStudyRequest object.
   ATL::CComPtr<ICQGCustomStudyRequest> m_spRequest;

   /// \brief Contains line time from CQGCEL when form is activated.
   const DATE m_LineTime;

private:

   /// \brief Request's baseExpression.
   CString m_BaseExpression;

   /// \brief Parameters' ListView control.
   CListCtrl m_lvParameters;

   /// \brief Parameter's new value EditBox control.
   CNumEdit m_edParameterValue;

   /// \brief Change parameter's value Button control.
   CButton m_btnChangeValue;

   /// \brief Request's range start date DateTimePicker control.
   CDateTimeCtrl m_dtpRangeStart;

   /// \brief Request's range start date.
   COleDateTime m_RangeStartDate;

   /// \brief Request's range start integer EditBox control.
   CNumEdit m_edRangeStart;

   /// \brief Request's range start SpinButton control.
   CSpinButtonCtrl m_spnRangeStart;

   /// \brief Request's range end date DateTimePicker control.
   CDateTimeCtrl m_dtpRangeEnd;

   /// \brief Request's range end date.
   COleDateTime m_RangeEndDate;

   /// \brief Request's range end date EditBox control.
   CNumEdit m_edRangeEnd;

   /// \brief Request's range end SpinButton control.
   CSpinButtonCtrl m_spnRangeEnd;

   /// \brief Request's type.
   int m_RequestType;

   /// \brief Request's include end CheckBox control.
   CButton m_chkIncludeEnd;

   /// \brief Request's include end.
   BOOL m_IncludeEnd;

   /// \brief Request's historical period ComboBox control.
   CComboBox m_cmbHistorical;

   /// \brief Request's intra day period Edit control.
   CNumEdit m_edIntraday;

   /// \brief Request's intraday period SpinButton control.
   CSpinButtonCtrl m_spnIntraday;

   /// \brief Request's continuation type ComboBox control.
   CComboBox m_cmbContinuation;

   /// \brief Request's equalize closes type CheckBox control.
   CButton m_chkEqualizeCloses;

   /// \brief Request's equalize closes.
   BOOL m_EqualizeCloses;

   /// \brief Request's days before expiration EditBox control.
   CNumEdit m_edDaysBeforeExp;

   /// \brief Request's days before expiration SpinButton control.
   CSpinButtonCtrl m_spnDaysBeforeExp;

   /// \brief Request's updates enabled.
   BOOL m_UpdatesEnabled;

   /// \brief Request's daily from intraday.
   BOOL m_DailyFromIntraday;

   /// \brief Request's sessions filter EditBox control.
   CNumEdit m_edSessionsFilter;

   /// \brief Request's sessions filter SpinButton control.
   CSpinButtonCtrl m_spnSessionsFilter;

   /// \brief Request's custom sessions filter.
   CString m_CustomSessionsFilter;

   /// \brief Request's custom sessions filter EditBox control.
   CEdit m_edCustomSessionsFilter;

   /// \brief Request's custom session.
   BOOL m_CustomSession;

   /// \brief Curves' ListView control.
   CListCtrl m_lvCurves;
   
   /// \brief Request's subscription level ComboBox control.
   CComboBox m_cmbSubscription;

   /// \brief Request's recalc period Label control.
   CStatic m_lblRecalcPeriod;

   /// \brief Request's recalc period EditBox control.
   CNumEdit m_edRecalcPeriod;

   /// \brief Request's recalc period SpinButton control.
   CSpinButtonCtrl m_spnRecalcPeriod;
};

enum CCustomStudyReqDlg::eRequestType
{
   rtIntInt,
   rtDateInt,
   rtDateDate
};

enum CCustomStudyReqDlg::eParametersColumns
{
   pcName,
   pcComment,
   pcType,
   pcDefaultValue,
   pcValue
};

enum CCustomStudyReqDlg::eCurvesColumns
{
   ccCheck,
   ccName
};